# -*- coding: utf-8 -*-
"""RELU_F_MNIST.ipynb

Automatically generated by Colaboratory.

Original file is located at
    https://colab.research.google.com/drive/14wpYhIl419UD8vjqm0iQRy4FNx1kT3h0
"""

from tensorflow.keras.datasets import fashion_mnist

# loads the Fashion-MNIST training and test dataset 
(x_train, y_train), (x_test, y_test)  = mnist.load_data()

# Our Class Names, when loading data from .datasets() our classes are integers
from tensorflow.python.client import device_lib

print(device_lib.list_local_devices())
from tensorflow.keras.preprocessing.image import ImageDataGenerator
from matplotlib import pyplot
from tensorflow.keras import backend as K

# Reshape our data to be in the format [number of samples, width, height, color_depth]
x_train = x_train.reshape(x_train.shape[0], 28, 28, 1)
x_test = x_test.reshape(x_test.shape[0], 28, 28, 1)

# Change datatype to float32
x_train = x_train.astype('float32')
x_test = x_test.astype('float32')
# Lets store the number of rows and columns
img_rows = x_train[0].shape[0]
img_cols = x_train[0].shape[1]

# store the shape of a single image 
input_shape = (img_rows, img_cols, 1)

# Normalize our data between 0 and 1
x_test /= 255.0
from tensorflow.keras.utils import to_categorical

# Now we one hot encode outputs
y_train = to_categorical(y_train)
y_test = to_categorical(y_test)

# Let's count the number columns in our hot encoded matrix 
print ("Number of Classes: " + str(y_test.shape[1]))

num_classes = y_test.shape[1]
num_pixels = x_train.shape[1] * x_train.shape[2]
from keras.activations import *
import numpy as np
from numpy.polynomial import Polynomial
from sklearn.datasets import make_blobs
from tensorflow.keras.utils import to_categorical,get_custom_objects
from keras.models import Sequential
from keras.layers import Dense
from matplotlib import pyplot
from os import makedirs
from sklearn.datasets import make_blobs
from sklearn.metrics import accuracy_score
from sklearn.linear_model import LogisticRegression
from keras.models import load_model
from numpy import dstack
import tensorflow as tf
from tensorflow.keras.datasets import mnist
from tensorflow.keras.models import Sequential 
from tensorflow.keras.layers import Dense, Dropout, Flatten
from tensorflow.keras.layers import Conv2D, MaxPooling2D, BatchNormalization
from tensorflow.keras import backend as K
from tensorflow.keras.optimizers import SGD 
from tensorflow.keras import regularizers
from keras.layers import DepthwiseConv2D, Reshape, Activation
import math


# fit model on dataset
def custom_activation(x):
    return (x**2) +x

get_custom_objects().update({'custom_activation': custom_activation})

L2 = 0.001
nets = 1
model = [0] *nets
for j in range(nets):
  model[j] = Sequential()
  model[j].add(Conv2D(32, kernel_size=(3, 3),
                 activation='relu',
                 kernel_regularizer = regularizers.l2(L2),
                 input_shape=input_shape))
  model[j].add(BatchNormalization())
  model[j].add(Conv2D(64, (3, 3), activation='relu', kernel_regularizer = regularizers.l2(L2)))
  model[j].add(BatchNormalization())
  model[j].add(MaxPooling2D(pool_size=(2, 2)))
  model[j].add(Dropout(0.2))
  model[j].add(Flatten())
  model[j].add(Dense(128, activation='relu',kernel_regularizer = regularizers.l2(L2)))
  model[j].add(Dropout(0.2))
  model[j].add(Dense(num_classes, activation='softmax'))

  model[j].compile(loss = 'mean_squared_error',
              optimizer = SGD(lr=0.01, momentum=0.9, clipnorm=1.0),
              metrics = ['accuracy'])

print( model[j].summary())
print('All Models Defined')
from keras.callbacks import ReduceLROnPlateau
learning_rate_reduction=ReduceLROnPlateau(monitor='val_accuracy', patience=3, verbose=0, factor=0.5, min_lr=0.00001)


# train for 20 epochs
history = [0] * nets
epochs = 10

train_datagen = ImageDataGenerator(
        rescale = 1./255,
        rotation_range=10,
        width_shift_range=0.1,
        height_shift_range=0.1,
        shear_range=0.1,
        zoom_range=0.1,
        horizontal_flip=True,
        fill_mode='nearest')

# Here we fit the data generator to some sample data.
#train_datagen.fit(x_train)

batch_size=32
# Fit the model
# Notice we use train_datagen.flow, this takes data & label arrays, generates batches of augmented data.
for j in range(nets):
    print(f'Individual Net : {j+1}')   
    history = model[j].fit(train_datagen.flow(x_train, y_train, batch_size = batch_size),
                              epochs = epochs,
                              validation_data = (x_test, y_test),
                              verbose = 1,
                              steps_per_epoch = x_train.shape[0] // batch_size)
model[j].evaluate(x_train, y_train, verbose=0)
model[j].evaluate(x_test,y_test, verbose=0)
import numpy as np
results = np.zeros((10000,10)) 

r=model[0].predict(x_test)
#print(y_test.shape)
#print(r.shape)
#print(results.shape)
for j in range(0,nets):
    
    results = results + model[j].predict(x_test)

res = np.argmax(results,axis = 1)
y_labels= np.argmax(y_test,axis = 1)
#print(res)
#print(y_test)

from sklearn.metrics import accuracy_score
from sklearn.metrics import precision_score
from sklearn.metrics import recall_score
from sklearn.metrics import f1_score
import numpy

y_test = numpy.array(y_test)
print(accuracy_score(res,y_labels ))
print ("Number of Nets: " + str(nets))

print ("Total accuracy: " + str(accuracy_score(res,y_labels )))

print ("Total accuracy: " + str(accuracy_score(res,y_labels )))
print ("Precision (average = None)" + str(precision_score(res,y_labels, average=None)))
print ("Recall (average = None)" + str(recall_score(res,y_labels, average=None)))
print ("F1 Score (average = None)" + str(f1_score(res,y_labels,  average=None)))
print ("Number of Nets: " + str(net))